varying vec2 texturePos;
varying vec2 positionInFacet;  ///< Ranges from (0, 0) to (1, 1)
uniform vec2 textureSize;
uniform float surfaceAspectRatio;
uniform vec2 facetOrigin;
uniform vec2 facetTopRight;
uniform vec2 paddingOrigin;
uniform vec2 paddingTopRight;

void main()
{
    // Find the position within the texture. At this point our coordinates are such that (-1,-1) to (1,1) map to the full surface.
    texturePos = vec2( ( gl_Vertex.x + 1.0 ) * textureSize.x / 2.0, ( gl_Vertex.y + 1.0 ) * textureSize.y / 2.0 );
    
    // Map vertex position to inside of padding.
    vec4 newVertex = gl_Vertex;
    newVertex.x *= ( paddingTopRight.x - paddingOrigin.x ) / 2.0;
    newVertex.y *= ( paddingTopRight.y - paddingOrigin.y ) / 2.0;
    newVertex.x += ( paddingOrigin.x + paddingTopRight.x ) / 2.0;
    newVertex.y += ( paddingOrigin.y + paddingTopRight.y ) / 2.0;
    
    // Then map to position with respect to this particular facet.
    vec2 facetVertex = vec2( ( newVertex.x + 1.0 ) / 2.0 , ( newVertex.y + 1.0 ) / 2.0 );
    facetVertex.x -= facetOrigin.x;
    facetVertex.y -= facetOrigin.y;
    facetVertex.x /= ( facetTopRight.x - facetOrigin.x );
    facetVertex.y /= ( facetTopRight.y - facetOrigin.y );
    positionInFacet = facetVertex;
    
    // Transform matrices are applied with respect to the whole facet, but within that transformed facet, we'll potentially be singling out a portion within padding to display.
    gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * newVertex;
}
